﻿using ELTE.ExamGenerator.Model;
using ELTE.ExamGenerator.ViewModel;

namespace ELTE.ExamGenerator;

public partial class App : Application
{
    IExamGeneratorModel? _model;
    ExamGeneratorViewModel? _viewModel;

    public App()
    {
        InitializeComponent();

        // model létrehozása
        _model = new ExamGeneratorModel(10, 0);
        _viewModel = new ExamGeneratorViewModel(_model);

        // nézetmodell létrehozása és befecskendezése a nézetbe
        MainPage = new AppShell(_viewModel);
    }

    protected override Window CreateWindow(IActivationState? activationState)
    {
        Window window = base.CreateWindow(activationState);

        window.Created += (s, e) =>
        {
            LoadState();
        };

        window.Resumed += (s, e) =>
        {
            LoadState();
        };

        window.Stopped += (s, e) =>
        {
            SaveState();
        };

        return window;
    }

    public void SaveState()
    {
        if (_model == null)
            return;

        Preferences.Set("questionCount", _model.QuestionCount); // elmentjük a tételek számát
        Preferences.Set("periodCount", _model.PeriodCount);

        // lekérdezzük a nem választható tételeket
        List<Int32> takenNumbers = new List<Int32>();
        for (Int32 number = 1; number <= _model.QuestionCount; number++)
        {
            if (!_model.IsTakeable(number))
                takenNumbers.Add(number);
        }

        Preferences.Set("takenNumbers", string.Join(',', takenNumbers)); // elmentjük a nem választható tételeket

        if (_model.IsGenerating) // ha fut a generálás
        {
            Preferences.Set("wasGenerating", true); // akkor ezt elmentjük
            _model.Take(); // leállítjuk a generálást
        }
        else
        {
            Preferences.Set("wasGenerating", false);
        }

        _model = null;
        _viewModel = null;
        (MainPage as AppShell)!.BindingContext = null;
        // a szemétgyűjtő kitörli a memóriából a modellt és a nézetmodellt
    }

    public void LoadState()
    {
        // ha el lett mentve az állapot, akkor visszatöltjük
        if (Preferences.ContainsKey("questionCount"))
        {
            _model = new ExamGeneratorModel(
                Convert.ToInt32(Preferences.Get("questionCount", 10)),
                Convert.ToInt32(Preferences.Get("periodCount", 0)));
            // az elmentett adatokkal példányosítjuk a modellt
        }
        else
        {
            _model = new ExamGeneratorModel(10, 0);
            // alapbeállítás a modellre
        }

        if (Preferences.ContainsKey("takenNumbers"))
        {
            var numberString = Preferences.Get("takenNumbers", "");

            if (numberString.Length > 0)
            {
                List<Int32> takenNumbers = numberString
                    .Split(',')
                    .Select(s => Convert.ToInt32(s))
                    .ToList();

                for (Int32 number = 1; number <= _model.QuestionCount; number++)
                {
                    _model.SetTakeable(number, !takenNumbers.Contains(number));
                }
            }
        }

        _viewModel = new ExamGeneratorViewModel(_model);
        (MainPage as AppShell)!.BindingContext = _viewModel;
        // nézetmodell létrehozása és befecskendezése a nézetbe

        // ha korábban futott a generálás, folytatjuk
        if (Preferences.ContainsKey("wasGenerating") && Preferences.Get("wasGenerating", false))
        {
            _model.Generate();
        }
    }
}
